@@ -14,3 +14,12 @@ pub fn listen_backoff() -> ExponentialBackoff {
         ..Default::default()
     }
 }
+
+pub fn run_control_chan_backoff() -> ExponentialBackoff {
+    ExponentialBackoff {
+        randomization_factor: 0.1,
+        max_elapsed_time: None,
+        max_interval: Duration::from_secs(1),
+        ..Default::default()
+    }
+}
